#include "GetConsoleConfig.h"
#include "IcomManager.h"
#include <avr/pgmspace.h>
#include <Arduino.h>
#include "PowerBandMap.h"

extern IcomManager icomMgr;
extern configEEProm ee;

void GetConsoleConfig::getConfig(const uint8_t modePin, const bool overRide)
{
	// if it is the first time ever,  then set the default computer baud
	// rate to 38400
	if (ee.data[offsetEEPROMData_ComputerBaudRateIndex] == 0xff)
		ee.data[offsetEEPROMData_ComputerBaudRateIndex] = 9;
	// and the default CI-V baud rate to 9600
	if (ee.data[offsetEEPROMData_CIVBaudRateIndex] == 0xff)
		ee.data[offsetEEPROMData_CIVBaudRateIndex] = 5;
	pinMode(modePin, INPUT);  // Pin 40 is used for configuration mode.
	digitalWrite(modePin,HIGH);  // activate the pull up resistor
	{  // see if the mode PIN is pulled low.  If not then
		uint8_t numhigh = 0;
		for (uint8_t i = 0 ; i < 10 ; ++i)
		{
			if (digitalRead(modePin) == HIGH) ++numhigh;
			if (numhigh >= 5)
			{
				digitalWrite(modePin,LOW);
				return;
			}
			delay(100);
		}
	}
	//  Defines the baud rate for configuration
	hs->begin(115200);
	hs->println();
	hs->println();
	hs->println();
	hs->println(F("Version 1.13 January 11, 2016"));
	hs->print(F("Free memory: "));
	hs->println(memoryFree());
/*
	for (uint8_t i = 0 ; i < 10 ; i++)  /// Dump the entire data...
	{
		hs->print(ee.data[i],HEX);
		hs->write(' ');
	}
*/
	while (true)  // this happens for ever
	{
		uint8_t item = getItemToChange();
		switch (item)
		{
		case 1:
			ee.data[offsetEEPROMData_ComputerBaudRateIndex] = getBaudRateIndex();
			break;
		case 2:
			ee.data[offsetEEPROMData_CIVBaudRateIndex] = getBaudRateIndex();
			break;
		case 3: //Transceive -- bit zero
			if (ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Transceive))
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_Transceive);
			else
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_Transceive);
			break;
		case 4: //Transceive -- USB
			if (ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_TransceiveUSB))
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_TransceiveUSB);
			else
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_TransceiveUSB);
			break;
		case 5: //Echo
			if (ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Echo))
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_Echo);
			else
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_Echo);
			break;
		case 6: //Subwitch
			if (ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Subwitch))
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_Subwitch);
			else
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_Subwitch);
			break;
		case 7: // Split off // Negative logic
			if (!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Split))
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_Split);
			else if (ee.data[offsetEEPROMData_ICOMRadioDeviceID] < 0xe0)
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_Split);
           else
            {
                printAdrWarn();
                 hs->println();
            }
			break;
		case 8: //PowerLevel  Negative logic!
			if (!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevel))
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevel);
			else if (ee.data[offsetEEPROMData_ICOMRadioDeviceID] < 0xe0) 
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevel);
            else
            {
                printAdrWarn();
                 hs->println();
            }
			break;        
		case 9: //PowerLevel Learn  Negative logic!
			if (!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevelLearn))
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevelLearn);
			else if (ee.data[offsetEEPROMData_ICOMRadioDeviceID] < 0xe0)
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevelLearn);
            else
            {
                printAdrWarn();
                 hs->println(F("learn"));
            }
			break;   
		case 10:  /// small or large power table.
			if (ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable))
				ee.bitClearEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable);
			else
				ee.bitSetEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable);
            // clean out the Power band data...
            for (byte i = offsetEEPROMData_PowerBandIndex ; i <  maxConfigEEPromByte ; ++i)
                ee.byteSetEE(i);
			hs->println(F("Resetting Power Band Map Table"));
			break;
        case 11: //Driver power table
            powerTable(overRide);
            break;
        case 12:  // ICOM Radio Device ID
            ee.data[offsetEEPROMData_ICOMRadioDeviceID] = changeRadioID();
            break;
        case 13:  // Reset defaults
            // clean out the Power band data...
            for (byte i = offsetEEPROMData_PowerBandIndex ; i <  maxConfigEEPromByte ; ++i)
                ee.byteSetEE(i);
            ee.byteSetEE(offsetEEPROMData_ICOMRadioDeviceID);
            ee.byteSetEE(offsetEEPROMData_flags);
 			ee.data[offsetEEPROMData_ComputerBaudRateIndex] = 9;
  			ee.data[offsetEEPROMData_CIVBaudRateIndex] = 5;
         break;
		default:
			break;
		};

		ee.update();
	}

}

void GetConsoleConfig::printAdrWarn()
{
    hs->print(F("=>> ERROR! MUST SET ICOM RADIO CI-V ADDRESS <=="));
}

void GetConsoleConfig::powerTable(const bool overRide)
{
     if (ee.data[offsetEEPROMData_ICOMRadioDeviceID] >= 0xe0)
    {
        printAdrWarn();
        return;
    }
	if (!bStarted)
    {
        bStarted = true;
        unsigned long CIVbaudRate = baudRateIndexToBaudRate(ee.data[offsetEEPROMData_CIVBaudRateIndex]);
        icomMgr.setupConsole(CIVbaudRate,overRide);
        hs->print(F("Setting CI-V Baud rate to "));
        hs->println(CIVbaudRate);
    }

    hs->println(F("IMPORTANT:\r\n\tWILL CHANGE POWER ON THE RADIO!\r\n\tRADIO MUST BE CONNECTED AND ON"));
    while (true)
    {
    uint8_t bandIndex = getBandToChangePower();
    if (bandIndex == 0xff)
        return;
    uint8_t newPower = changePower(~ee.data[offsetEEPROMData_PowerBandIndex+bandIndex]);
    ee.data[offsetEEPROMData_PowerBandIndex+bandIndex] = ~newPower;
    }
}

uint8_t GetConsoleConfig::changePower(const byte initialValue)
{
    byte increment = 1;
    byte value = initialValue;
    hs->println(F("+ increase\r\n- decrease\r\n1-9 change increment\r\nAny key to exit\r\n"));
    while (true)
    {
        hs->print(F("Current Value = "));
        hs->print(value,DEC);
        printByteAsPerCent(value);
        icomMgr.sendPowerMessage(value);
        while (!hs->available())  //wait until a key is hit.. 
        {
            icomMgr.loopTX(); // send out messages   
        }
        byte ret = hs->read();
        if (ret >= '1' && ret <= '9')
        {
            increment = ret - '0';
            hs->print(F("Current increment = "));
            hs->println(increment,DEC);
            continue;
        }
        if (ret == '+')
        {
            uint16_t newValue = value + increment;
            if (newValue > 255) 
                newValue = 255;
            value = static_cast<byte>(newValue);
            continue;
        }
        if (ret == '-')
        {
            if (increment > value)
                value = 0;
            else
                value -= increment;
            continue;
        }
        break;
    }
    return value;
}

uint8_t GetConsoleConfig::changeRadioID()
{
    while (true)
    {
		hs->print(F("Enter radio CI-V address (to enter hex, place 0x before value - eg 0xa3)? "));
		uint8_t v = getByte();
		if (v < 0xe0) 
          return v;
		hs->println();
		hs->println(F("Icom radio CI-V address must be less than 0xE0"));
    }
    return 0;  // should never happen
}

uint8_t GetConsoleConfig::getBandToChangePower()
{

        uint8_t index = 0;
        uint8_t bandIndex = 0;
        uint16_t lowKHz = 0;
        uint16_t highKHz = 0;
        while (true)
        {
            while (true) {
                bandIndex = getPowerBandData(index,lowKHz,highKHz);
                ++index;
                if (bandIndex == 0xff)
                    continue;
                if (bandIndex ==  0xfe)
                    break;

                hs->print(bandIndex+1);
                hs->print(F(" - "));
                hs->print(lowKHz);
                hs->print(F("KHz -> "));
                hs->print(highKHz);
                hs->print(F("KHz = "));
                uint8_t value = ~ee.data[offsetEEPROMData_PowerBandIndex+bandIndex];
                hs->print(value,DEC);
                printByteAsPerCent(value);
            }
            hs->print(F("Select band  1.."));
			uint8_t numberPowerBand = getNumberPowerBand();
            hs->print(numberPowerBand+1,DEC);
            hs->print(F("  0 or Enter to Return and Save: ? "));
            index = getByte();
			hs->println();
            if (!index)
                return 0xff;
            --index;
            if (index <= numberPowerBand)
                break;
        }
        hs->println();
        
        return index;
}

void GetConsoleConfig::printByteAsPerCent(const uint8_t value)
{
    uint16_t value10 =  static_cast<uint16_t>( (static_cast<uint32_t>(value) * 1000) / 255);
    uint8_t valuea = value10/10;
    uint8_t valueb = value10%10;
    hs->print(F(" "));
    hs->print(valuea,DEC);
    hs->print(F("."));
    hs->print(valueb,DEC);
    hs->println(F("%"));
}
uint8_t GetConsoleConfig::getBaudRateIndex(void)
{
	uint8_t index = 0;
	while (true)
	{
		hs->println(F("Select baud rate:"));
		for (uint8_t i = 1 ; i < 12 ; ++i)
		{
			unsigned long br = baudRateIndexToBaudRate(i);
			hs->print(F("  "));
			hs->print(i);
			hs->print(F("- ") );
			hs->println(br);
		}
		hs->print(F("Which baud rate 1..11? ") );
		index = getByte();
		if (index >= 1 && index <= 11) break;
		hs->println(F("Try again!"));
	}
	hs->println();
	return index;
}

uint8_t GetConsoleConfig::getItemToChange()
{
	while (true)
	{
		hs->println();
		hs->println(F("Config CI-V Router"));
		hs->print(F("1 -Computer baud rate = "));
		hs->print(baudRateIndexToBaudRate(ee.data[offsetEEPROMData_ComputerBaudRateIndex]));
		hs->println();

		hs->print(F("2 -CI-V baud rate = "));
		hs->print(baudRateIndexToBaudRate(ee.data[offsetEEPROMData_CIVBaudRateIndex]));
		hs->println();

		hs->print(F("3 -Transceive msg to other radios = "));
		printTrueOrFalse(ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Transceive));
		
		hs->print(F("4 -Transceive msg to USB = "));
		printTrueOrFalse(ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_TransceiveUSB));

		hs->print(F("5 -USB Echo = "));
		printTrueOrFalse(ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Echo));

		hs->print(F("6 -Subswitch = "));
		printTrueOrFalse(ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Subwitch));

		hs->print(F("7 -Split OFF when QSY >1MHz = "));  // 
		printTrueOrFalse(!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Split));

		hs->print(F("8 -Enable driver power level = "));  //negative logic
		printTrueOrFalse(!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevel));

 		hs->print(F("9 -Enable driver power level learn = "));  //negative logic
		printTrueOrFalse(!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevelLearn));

		hs->print(F("10-Small power table = "));
		printTrueOrFalse(ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable));

        hs->println(F("11-Driver power table"));

        hs->print(F("12-Icom radio CI-V address "));
        uint8_t icomID = ee.data[offsetEEPROMData_ICOMRadioDeviceID];
        if (icomID < 0xe0)
        {
            hs->print(F("0x"));
            printnibblehex(icomID >> 4);
            printnibblehex(icomID & 0x0f);
            hs->println(F(" hex"));
        }
        else
            hs->println(F("UNCONFIGURED"));
        
        hs->println(F("13-Reset all to default values"));

		hs->print(F("Which parameter to change? "));
		uint8_t retValue = getByte();
		hs->println();
		hs->println(retValue,DEC);
		if (retValue > 0 && retValue < 14) return retValue;
		hs->println(F("Out of Range"));
	}
	return 0;
}

void GetConsoleConfig::printTrueOrFalse(const bool bx)
{
	if (bx)
		hs->println(F("True"));
	else
		hs->println(F("False"));
}

void GetConsoleConfig::printnibblehex(const uint8_t nibble)
{
    if (nibble < 10)
        hs->write(nibble+'0');
    else
        hs->write(nibble-10+'A');
}

const PROGMEM uint32_t BaudRateTable[] = {
	0, 300, 1200, 2400, 4800, 9600, 14400, 19200, 28800, 38400, 57600UL, 115200UL };

uint32_t baudRateIndexToBaudRate(uint8_t index)
{
	/// 12 of them in all 
  if (index > sizeof(BaudRateTable)/4) return 0;
  return pgm_read_dword(BaudRateTable+index);
}

extern int __bss_end;
extern void * __brkval;
int memoryFree(void)
{
	int freeValue;
	if((int)__brkval == 0)
		freeValue = ((int)&freeValue) - ((int)&__bss_end);
	else
		freeValue = ((int)&freeValue) - ((int)__brkval);
	return freeValue;
}
