#ifndef __ARDUINO_GetConsoleConfig_H
#define __ARDUINO_GetConsoleConfig_H

#include "GetConsole.h"
#include "ConfigEEProm.h"

const uint8_t offsetEEPROMData_PowerBandIndex = 7;  // uses the next 43 values
const uint8_t offsetEEPROMData_ICOMRadioDeviceID = 6;
const uint8_t offsetEEPROMData_ComputerBaudRateIndex = 5;
const uint8_t offsetEEPROMData_CIVBaudRateIndex = 3;
const uint8_t offsetEEPROMData_flags = 4;

//Bit mapping for Configuration true flase flags within the offsetEEPROMData_flags byte
const uint8_t ConfigurationFlagBit_Split = 7;  // Auto split off
const uint8_t ConfigurationFlagBit_SmallPowerTable = 6;  // small power table or large?
const uint8_t ConfigurationFlagBit_PowerLevelLearn = 5; // /// This is inverse  1 means no
const uint8_t ConfigurationFlagBit_PowerLevel = 4;  /// This is inverse  1 means no
const uint8_t ConfigurationFlagBit_TransceiveUSB = 3; //Transceive msg to USB
const uint8_t ConfigurationFlagBit_Subwitch = 2; // Subswitch
const uint8_t ConfigurationFlagBit_Echo = 1; //USB Echo
const uint8_t ConfigurationFlagBit_Transceive = 0;  //Transceive msg to other radios

class GetConsoleConfig : public GetConsole
{
public:
	GetConsoleConfig(HardwareSerial * hsx) : 
      GetConsole(hsx),
      bStarted(false)
      {}
	virtual ~GetConsoleConfig(void) {}
	// never returns from here if we are in this mode...
	void getConfig(const uint8_t modePin, const bool overRide = false);

private:
    void printAdrWarn();
    bool bStarted;  // have we started the CI-V serial ports?
    void powerTable(const bool overRide = false);
    uint8_t getBandToChangePower();
    uint8_t changePower(const byte initialValue);
	uint8_t getBaudRateIndex(void);
	uint8_t getItemToChange();
	void printTrueOrFalse(const bool bx);
    uint8_t changeRadioID();
    // puts out " XXX.X%\r\n"
    void printByteAsPerCent(const uint8_t value);
    void printnibblehex(const uint8_t nibble);
};

	/// @return if 0,  then use the default or else dunno
extern unsigned long baudRateIndexToBaudRate(uint8_t index);
extern int memoryFree(void);

#endif // __ARDUINO_GetConsoleConfig_H

