#ifndef ARDUINO_ICOMMANAGER_H_
#define ARDUINO_ICOMMANAGER_H_

#include <Arduino.h>
#include "SimpleTimeout.h"
#include "IcomDeviceStatus.h"
#include "IcomSerial.h"
#include "IcomMessage.h"
#include "ObjectPool.h"
#include "GetConsoleConfig.h"

extern const bool bThreePorts;

const unsigned long timeOutBroadcast = 3000;
/// Max number of outstanding Icom Messages,  either being read or in queues
const byte numberOfIcomMesssages = 30;

/**
 * @author James Michener K9JM  (c) 2012
 *
 */

class IcomManager : public IcomDeviceStatus
{
public:
	IcomManager(void) : 
	  loopCounter(0) , ConfigurationFlags(0xff)  {
		  stBroadcast.start(timeOutBroadcast);
	  }
	  ~IcomManager(void) {
	  }
	  void onMessage( IcomSerial * is);

	  void setup(const unsigned long baudRateComputer, const unsigned long baudRateCIV, const uint8_t configFlags, const bool bOverRide = false);
	  void loop(void);
      void loopTX(void);
	  void checkBroadcast(void);
	  void sendBroadcast(const boolean bFreq, const boolean bMode);
	  inline byte getLoopCounter(void) const { return loopCounter; }
      inline IcomSerial * p3Adr()  { return &p3; }

      void setupConsole( const unsigned long baudRateCIV, const bool bOverRide = false);
      void sendPowerMessage(const byte value);
      void sendPowerQueryMessge();
	  void sendSplitOffMessage();
	  void sendMainReadoutMessage();
	  void sendFreqMessage(const unsigned long freq);
private:
	SimpleTimeout stBroadcast;
	IcomMessage * getMTY(void);
	void makeAndSendBroadcast(IcomSerial * px, const byte index, const boolean bFreq, const boolean bMode);
	void onMessageWork( IcomSerial * is);
	void copyAndSendMessage(IcomSerial & p, IcomMessage * msg);
	ObjectPool<IcomMessage,numberOfIcomMesssages> icomMessagePool;
	IcomSerial p1;
	IcomSerial p2;
	IcomSerial p3;
	IcomSerial p0; /// computer port
	byte loopCounter;
	byte ConfigurationFlags;
};


#endif // ARDUINO_ICOMMANAGER_H_

