
#include "IcomSerial.h"
extern const bool bThreePorts;
extern void onMessageGlobal( IcomSerial * is);

void IcomSerial::checkReceiveThrowAway()
{
    hs->read();
}
void
IcomSerial::checkReceive(void)
{
	int iRead = hs->read();
	if (iRead < 0 || !imReceive ) return;
	byte bin = static_cast<byte> (iRead);
	byte res = add(bin);
	if (res <= 1) return;  // nothing to do
	if (res == 3)
	{
		// if we are on a single (simplex) line then throw away the message
		if (!flagRead(isDuplexFlag) && imTransmit)
		{
			/// check receive message against the transmit message and if there
			/// is a problem we can try again.
			//      if (imTransmit->isEqual(*imReceive) )
			{
				imTransmit->setMTY();
				imTransmit = NULL;
				//       imReceive->setInUse();
			}
			/*
			else
			{
			queueIcomMessage(imTransmit);
			Serial3.println(F("NEqual"));
			}
			*/
		}
		else 
			if (imReceive->ValidateReceiveMessage())
				::onMessageGlobal(this);
            else 
            {
                if (!bThreePorts)
		        {
                    Serial3.print(F(" Invalid msg: "));
                    imReceive->dump();
                }
                imReceive->setMTY();
                return;
		    }
	}
	imReceive->setInUse();
}


void
IcomSerial::checkTransmit(void)
{
	// if valid start flag and simplex and not transmitting, 
	// then don't bother checking to see if there is message
	// is available
	if ((flags & 7) == 1 )
	{
		flagSet(isTXHoldOff);
		stTransmit.start(4);
		return;
	}
	if (flagRead(isTXHoldOff))
	{
		if (!stTransmit.isTimedout()) return;
		flagClear(isTXHoldOff);
	}
	if (!imTransmit)
	{
		// if true then there is nothing to send
		if (!transmitMessageQueue.isAvailable()) return;
		imTransmit = transmitMessageQueue.get();
		/// send out the first of the two start bytes
		hs->write(icomMessageStartByte);

		if (flagRead(isDuplexFlag))
		{
			hs->write(imTransmit->buffer,imTransmit->m_Counter);
//            imTransmit->dumpSpecial();
			imTransmit->setMTY();
			imTransmit = NULL;
			return;
		}

		hs->write(icomMessageStartByte);
		m_transmitOffset = 1;
		flagSet(isTransmittingFlag);
		stTransmit.start(icomMessageTransmitTimeOutinMS);
		return;
	}
	/// send from the actual message
	if (flagRead(isTransmittingFlag))
	{
		hs->write(imTransmit->buffer[m_transmitOffset]);
		++m_transmitOffset;
		if (m_transmitOffset >= imTransmit->m_Counter)
		{
			flagClear(isTransmittingFlag);
			if (flagRead(isDuplexFlag))
			{
				imTransmit->setMTY();
				imTransmit = NULL;
			}
			m_transmitOffset = 0;
		}
	}
	else  // waiting to for receive message to clear imTransmit
	{
		if (stTransmit.isTimedout())
		{
			imTransmit->setMTY();
			imTransmit = NULL;
		}
	}
}

byte
IcomSerial::add(const byte x)
{
	byte bRet = 0;

	if (stReceive.isTimedout())
	{
		//  on time out
		imReceive->setInUse();  /// sets m_Counter = 0xff
		flagClear(isValidStartFlag);
		bRet = 1;
	}

	if ( (m_lastin == icomMessageStartByte) && (x == icomMessageStartByte) )
	{
		flagSet(isValidStartFlag);
		imReceive->setMTY(); // sets m_Counter = 0   but it will increment 2 lines down
	}
	m_lastin = x;
	if (! imReceive->add(x))
	{
		flagClear(isValidStartFlag);
		bRet = 2;
	}
	if ( x == icomMessageStopByte)
	{
		bRet = flagRead(isValidStartFlag) ? 3 : 4;
		flagClear(isValidStartFlag);
	}

	stReceive.start(icomMessageReceiveTimeOutinMS);
	return bRet;
}

void
IcomSerial::queueIcomMessage( IcomMessage * x)
{
	if (!x) return; // sanity check
	if (x->isEmpty()) return;
	if (x->isMarkedInUse())
	{
		x->setMTY();
		return;
	}
	if (transmitMessageQueue.check())
		transmitMessageQueue.put(x);
	else
	{
		if (!bThreePorts)
		{
			Serial3.write(devID);
			Serial3.println(F(" SNARK!"));
		}
		x->setMTY();  /// throws the message away
	}
}
