#ifndef ARDUINO_OBJECTPOOL
#define ARDUINO_OBJECTPOOL
#include <Arduino.h>

/**
* @author James Michener (c) 2012
* Very very simple templated class of object pool
* Creates a pool of objects and gives you a pointer to
* the next object in the pool in a circular queue.
* Does no checking if object is still in use. 
* If one uses this class, then the user must check
* if "in use" is a criteria.
*
* Only real routine here is getNext()
*/
template <class x, byte sizex> class ObjectPool
{
public:
	ObjectPool(void) : index(0)
	{}
	~ObjectPool(void) {}
	// it may or may not be in use... user choice
	inline x * getNext(void) {
		x * ret = &buffer[index]; 
		++index;
		if (index >= sizex) index = 0;
		return ret; }
private:
	byte index;
	x buffer[sizex];
};

#endif //  ARDUINO_OBJECTPOOL
