#include <avr/pgmspace.h>
#include <Arduino.h>
#include "ConfigEEProm.h"
#include "GetConsoleConfig.h"
#include "IcomManager.h"

extern configEEProm ee;
extern IcomManager icomMgr;

static const int8_t undefinedBand = 0xff;
/**
 * Values in the tables are in KHz and repsents the band edge for antenna selection
 * Note:  The highest frequency is 64MHz
 * The most number of bands is 256,  since the index is that value
 */


const PROGMEM uint16_t PowerBandFreqTableF[] = {
 0 , 1800, 1812, 1824, 1836, 1848, 1860, 1872, 1884, 1896, 1908, 1920, 1932, 1944, 1956, 1968, 1980, 1992, 2000,
 3500, 3525, 3550, 3575, 3600, 3625, 3650, 3675, 3700, 3725, 3750, 3775, 3800, 3825, 3850, 3875, 3900, 3925, 3950, 3975, 4000,
 5330, 5370, 5410,
 7000, 7050, 7100, 7150, 7200, 7250, 7300,
 10100, 10150,
 14000, 14050, 14100, 14150, 14200, 14250, 14300, 14350,
 18068, 18118, 18168,
 21000, 21075, 21150, 21225, 21300, 21375, 21450,
 24890, 24940, 24990,
 28000, 28150, 28300, 28450, 28600, 28750, 28900, 29050, 29200, 29350, 29500, 29650, 29700,
 50000, 50500, 51000, 51500, 52000, 52500, 53000, 53500, 54000,
0xffff };

const PROGMEM int8_t BandIndexF[] = { undefinedBand,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, undefinedBand,
 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, undefinedBand,
 37, 38, undefinedBand,
 39, 40, 41, 42, 43, 44, undefinedBand,
 45, undefinedBand,
 46, 47, 48, 49, 50, 51, 52, undefinedBand,
 53, 54, undefinedBand,
 55, 56, 57, 58, 59, 60, undefinedBand,
 61, 62, undefinedBand,
 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, undefinedBand,
 75, 76, 77, 78, 79, 80, 81, 82, undefinedBand,
undefinedBand, undefinedBand,undefinedBand };


const PROGMEM uint16_t PowerBandFreqTableC[] = {
 0 , 1800, 1900, 2000,
 3500, 3675, 3850, 4000,
 5330, 5410,
 7000, 7150, 7300,
 10100, 10150,
 14000, 14200, 14350,
 18068, 18168,
 21000, 21250, 21450,
 24890, 24990,
 28000, 28500, 29000, 29500, 29700,
 50000, 51000, 52000, 53000, 54000,
0xffff };

const PROGMEM int8_t BandIndexC[] = { undefinedBand,
 0, 1, undefinedBand,
 2, 3, 4, undefinedBand,
 5, undefinedBand,
 6, 7, undefinedBand,
 8, undefinedBand,
 9, 10, undefinedBand,
 11, undefinedBand,
 12, 13, undefinedBand,
 14, undefinedBand,
 15, 16, 17, 18, undefinedBand,
 19, 20, 21, 22, undefinedBand,
undefinedBand, undefinedBand, undefinedBand };

 // remember to set numberPowerBand in PowerBandMap.h

uint8_t getNumberPowerBand()
{
	return ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable) ? 22 : maxNumberPowerBand -2 ;
}

uint8_t getsizeofPowerBandFreqTable()
{
	return ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable) ? sizeof(PowerBandFreqTableC) : sizeof(PowerBandFreqTableF) ;
}

uint8_t getsizeofPowerBandIndex()
{
	return ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable) ? sizeof(BandIndexC) : sizeof(BandIndexF) ;
}

const PROGMEM uint16_t * getPowerBandFreqTable()
{
	return ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable) ? PowerBandFreqTableC : PowerBandFreqTableF;
}

const PROGMEM int8_t * getBandIndex()
{
		return ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_SmallPowerTable) ? BandIndexC : BandIndexF;
}

byte
lookupPowerBandIndex ( const uint16_t value, const byte guessIndex)
{
	uint8_t sizeofPT = getsizeofPowerBandFreqTable();
	uint8_t Ret = (guessIndex != 0xff) ? guessIndex : sizeofPT / 4;
	uint8_t lowIndex = 0;
	uint8_t highIndex = sizeofPT / 2;

	while (true)
	{
		uint16_t tableValueHigh = pgm_read_word( getPowerBandFreqTable() + Ret + 1);
		uint16_t tableValueLow = pgm_read_word( getPowerBandFreqTable() + Ret);
		if (tableValueLow > value )
			highIndex = Ret;
		else if ( tableValueHigh <= value )
			lowIndex = Ret + 1;
		else
			break;
		Ret = static_cast<uint8_t> ((static_cast<int16_t>(highIndex) + lowIndex) / 2);
		continue; 
	}
	return static_cast<byte>(Ret);
}

byte
freqToPowerBandIndex(const unsigned long freq, const byte guessIndex)
{
	return lookupPowerBandIndex( static_cast<uint16_t>(freq/1000), guessIndex);
}

byte
PowerBandIndexToBandIndex(const byte index)
{
	return pgm_read_byte(getBandIndex()+index);
}

byte getPowerBandData(const byte index, uint16_t & lowKHz, uint16_t & highKHz)
{
    if (index >= getsizeofPowerBandIndex())
        return 0xfe;
    const  uint16_t * pbft = getPowerBandFreqTable();
    lowKHz = pgm_read_word( pbft + index);
    highKHz = pgm_read_word( pbft + index + 1);
    return PowerBandIndexToBandIndex(index);
}


byte LastPowerBandIndex = 0xff;  // this is the initial guess at start up.
byte LastPowerIndex = 0xff;
uint16_t lastFrequency = 0;

void PowerBandOnNewFreq(const unsigned long freq)
{
    // see if an Icom Radio Device ID has been established in the configuration
    if (ee.data[offsetEEPROMData_ICOMRadioDeviceID] >= 0xe0)
        return;

	if (!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_Split))
	{
		uint8_t radioID = ee.data[offsetEEPROMData_ICOMRadioDeviceID];
		uint8_t index = icomMgr.findOrigID(radioID);
		if (index != 0xff)
		{
			uint16_t freqkhz = freq/1000;
			int16_t deltaFreq = lastFrequency - freqkhz;
			if (deltaFreq < 0) 
				deltaFreq = -deltaFreq;
			if (deltaFreq >= 1000)
			{
				icomMgr.sendSplitOffMessage();
				icomMgr.sendMainReadoutMessage();
				icomMgr.sendFreqMessage(freq);
				icomMgr.icomDev[index].main.freq = freq;
//				icomMgr.icomDev[index].sub.freq = freq;
//				Serial3.println(F("SPLIT"));
				icomMgr.sendBroadcast(true,false);
			}
			lastFrequency = freqkhz;
		}
	}
    // see if either the power level or power level learn has been enabled
    const byte plenable = ConfigurationFlagBit_PowerLevelLearn | ConfigurationFlagBit_PowerLevel;
    if ((ee.data[offsetEEPROMData_flags] & plenable) == plenable)
        return;
    // now I have to pay attention to the new frequency
    // Are we on a new band?
    byte CurrentPowerBandIndex = freqToPowerBandIndex(freq,LastPowerBandIndex == 0xff ? 20 :LastPowerBandIndex);
    if (LastPowerBandIndex == CurrentPowerBandIndex)
        return;

    /** Test Code
    {
        uint16_t low(0);
        uint16_t high(0);
        getPowerBandData(LastPowerBandIndex,low, high);
        Serial3.print(F("NB: "));
        Serial3.print(low);
        Serial3.print(F("->"));
        Serial3.println(high);
    }
    */

    // Remember these are negative logic!
    // if power level learn is enabled, then ask the radio the RF POWER knob position
    if (LastPowerBandIndex != 0xff)  // not at start up!!
    {
    if (!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevelLearn))
       {
            LastPowerIndex = PowerBandIndexToBandIndex(LastPowerBandIndex);
            icomMgr.sendPowerQueryMessge();
            // this is a magic number of checks... 1000 is too small 1500 works
            // going with 3000
            for (int i = 0 ; i < 3000 ; ++i)
                icomMgr.loop();
        }
    }
    // if power level is enabled.... send the message to the radio
    if (!ee.bitReadEE(offsetEEPROMData_flags,ConfigurationFlagBit_PowerLevel))
    {
        byte currentBandIndex = PowerBandIndexToBandIndex(CurrentPowerBandIndex);
        // again it is the 1's complement of the data actually stored.
        uint8_t value = ~ee.data[offsetEEPROMData_PowerBandIndex+currentBandIndex];
        icomMgr.sendPowerMessage(value);
    }
    LastPowerBandIndex = CurrentPowerBandIndex;
}

void PowerBandOnMessage(IcomMessage * imp)
{
//    imp->dumpSpecial();
    if (!imp->isPower() || imp->m_Counter != 8)
        return;
    byte value = ~imp->getByteResponse();
//    Serial3.print(F("Read pwr: 0x"));
 //   Serial3.println(value,HEX);
    ee.data[offsetEEPROMData_PowerBandIndex+LastPowerIndex] = value;
}
